<?php
/**
 * @version   1.1 
 * @author    Dovi42 
 * @home      http://dovi42.hu/
 * @package   Image library
 * @license   All Rights Reserved - Minden jog fenntartva
 * @copyright (C) 2020 Dovalovszki Tamás - All rights reserved.
 **/

// Check to ensure this file is within the rest of the framework

if (!defined('DS')) { define( 'DS', DIRECTORY_SEPARATOR ); }

class JDovi42Image extends stdClass
{
    private $_watermark_image = 'images/watermark.png';
    private $_watermark_text  = null;
    private $_align           = array('vertical' => 'center',      // Függőleges : top, center, bottom
                                      'horizontal' => 'center'); // Vízszintes : left, center, right
    private $_gd = false; 
    
    public $image = array('filename' => null,
                          'title'    => null,
                          'width'    => 0,
                          'height'   => 0,
                          'name'     => null,
                          'path'     => null,
                          'ext'      => null,
                          'filesize' => null,
                          'created'  => null,
                          'thumb'    => '0',
                          'thumbDir' => '_thumbs'); 
                          
    public $output = array();
    public $autoLandscape = true;
    public $jpeg_compress = 85;

    public $watermark = array('image'  => null,
                              'width'  => 0,
                              'height' => 0,
                              'walign' => 'left',
                              'halign' => 'bottom',
                              'text'   => null);

	/**
	 * JImage::__construct()
	 * Alapértelemezz beállításokkal létrehozza a kép objektumot
	 * 
	 * @param mixed $fid
	 * @return
	 */
	public function __construct($image)
	{

        if (extension_loaded('gd')) {
            $this->_gd = true;
        }        

        $parts = pathinfo($image);
        
        $this->image['filename'] = $parts['basename'];
        $this->image['name']  = $parts['filename'];
        $this->image['title'] = $this->image['name'];
        $this->image['ext']   = $parts['extension'];
        $this->image['path']  = $parts['dirname'];

        $size = getimagesize($this->image['path'] .DS. $this->image['filename']);
        $this->image['width']  = $size[0];
        $this->image['height'] = $size[1];
        
        $this->image['filesize'] = filesize($this->image['path'] .DS. $this->image['filename']);
        $this->image['created'] = date ("Y-m-d H:i:s", filemtime($this->image['path'] .DS. $this->image['filename']));
        
        $this->output = $this->image;
        
        $this->watermark['image'] = str_replace(DIRECTORY_SEPARATOR . 'libs', '', dirname(__FILE__)) .DS. $this->_watermark_image;
        $this->watermark['text'] = $this->_watermark_text;
        $size = @getimagesize($this->watermark['image']);
        $this->watermark['width'] = $size[0];
        $this->watermark['height'] = $size[1];
    }

    function setAlign($horizontal = 'center', $vertical = 'center') {
	   $this->_align['horizontal'] = $horizontal;
	   $this->_align['vertical'] = $vertical;
    }


    function normalizeName()
    {
        $this->image['ext'] = strtolower($this->image['ext']);
        $str = trim(mb_strtolower($this->image['name'], 'UTF-8'));

        $accented = array( "á", "é", "í", "ó", "ö", "ő", "ú", "ü", "ű" );  
        $mask = array( "a", "e", "i", "o", "o", "o", "u", "u", "u" );  
        $str = str_replace( $accented, $mask, $str );  

        $str = str_replace('-', ' ', $str);
        $str = str_replace('.', '', $str);
        $str = preg_replace(array('/\s+/','/[^A-Za-z0-9\-]/'), array('-',''), $str);
        $this->image['name'] = $str;
        
        rename( $this->image['path'] .DS. $this->image['filename'], $this->image['path'] .DS. $this->image['name'] . '.' . $this->image['ext'] );
        $this->image['filename'] = $this->image['name'] . '.' . $this->image['ext']; 
    }

    function rotateImage($mode = 'left')
    {
        if ($mode == 'left') {
            $degrees = 90;
        } else {
            $degrees = 270;
        }
        switch (strtolower($this->image['ext']))
        { 
        	case 'jpg':
               $source = imagecreatefromjpeg( $this->image['path'] .DS. $this->image['filename'] );
        	   break;
        	case 'gif':
               $source = imagecreatefromgif( $this->image['path'] .DS. $this->image['filename'] );
               imagealphablending( $source, true );
               imagesavealpha( $source, true );
        	   break;
        	case 'png':
               $source = imagecreatefrompng( $this->image['path'] .DS. $this->image['filename'] );
               imagealphablending( $source, true );
               imagesavealpha( $source, true );
        	   break;
        }
        $rotated = imagerotate($source, $degrees, 0);
        switch (strtolower($this->image['ext']))
        { 
        	case 'jpg':
               imagejpeg($rotated, $this->image['path'] .DS. $this->image['filename'], $this->jpeg_compress);
        	   break;
        	case 'png':
               imagepng($rotated, $this->image['path'] .DS. $this->image['filename'], floor($this->jpeg_compress / 10));
        	   break;
        	case 'gif':
               imagegif($rotated, $this->image['path'] .DS. $this->image['filename']);
        	   break;
        }
        imagedestroy($source);
        imagedestroy($rotated);
        $size = getimagesize($this->image['path'] .DS. $this->image['filename']);
        $this->image['width']  = $size[0];
        $this->image['height'] = $size[1];
        
        $this->image['filesize'] = filesize($this->image['path'] .DS. $this->image['filename']);
    }

    function getOrientation($size, $target)
    {
        $size = explode('x', $size);
        $landscape = true;
        if ((int)$size[1] > (int)$size[0]) {
            $landscape = false;
        }
        $target = explode('x', $target);
        $t_landscape = true;
        if ((int)$target[1] > (int)$target[0]) {
            $t_landscape = false;
        }
        
        if ($landscape != $t_landscape) {
            return $target[1].'x'.$target[0];
        }
        return $target[0].'x'.$target[1];
    }

    function createWebP($dir = '')
    {
        if ($dir != '') { $dir .= DS; }
        
        switch (strtolower($this->image['ext']))
        { 
        	case 'jpg':
               $source = imagecreatefromjpeg( $this->image['path'] .DS. $dir . $this->image['filename'] );
        	   break;
        	case 'gif':
               $source = imagecreatefromgif( $this->image['path'] .DS. $dir . $this->image['filename'] );
               imagealphablending( $source, true );
               imagesavealpha( $source, true );
        	   break;
        	case 'png':
               $source = imagecreatefrompng( $this->image['path'] .DS. $dir . $this->image['filename'] );
               imagealphablending( $source, true );
               imagesavealpha( $source, true );
        	   break;
        }
        ob_start();
        imagejpeg($source, NULL, 100);
        $cont =  ob_get_contents();
        ob_end_clean();
        imagedestroy($source);

        $content =  imagecreatefromstring($cont);
        imagewebp($content, $this->image['path'] .DS. $dir . $this->image['name'].'.webp', $this->jpeg_compress);
        imagedestroy($content);        
    }

    
    function resizeImage($new_size, $outPath = null)
    {
        $new = $this->getOrientation($this->image['width'].'x'.$this->image['height'], $new_size);
        $new = explode('x', $new);
        $size = array(
                'w' => $new[0],
                'h' => $new[1]
            );
        $percent['w'] = $size['w'] / $this->image['width']; 
        $percent['h'] = $size['h'] / $this->image['height'];
        if ($percent['w'] < $percent['h']) {
            $percent = $percent['w']; 
        } else {
            $percent = $percent['h'];
        } 

        if ($percent < 1) { 
            $new_width  = floor( $this->image['width'] * $percent );
            $new_height = floor( $this->image['height'] * $percent );
            
            $thumb = imagecreatetruecolor($new_width, $new_height);
            switch (strtolower($this->image['ext']))
            { 
            	case 'jpg':
                   $source = imagecreatefromjpeg( $this->image['path'] .DS. $this->image['filename'] );
            	   break;
            	case 'gif':
                   $source = imagecreatefromgif( $this->image['path'] .DS. $this->image['filename'] );
                   imagealphablending( $source, true );
                   imagesavealpha( $source, true );
            	   break;
            	case 'png':
                   $source = imagecreatefrompng( $this->image['path'] .DS. $this->image['filename'] );
                   imagealphablending( $source, true );
                   imagesavealpha( $source, true );
                   
            	   break;
            }
            // imagecopyresized($thumb, $source, 0, 0, 0, 0, $new_width, $new_height, $this->image['width'], $this->image['height']);
            imagecopyresampled($thumb, $source, 0, 0, 0, 0, $new_width, $new_height, $this->image['width'], $this->image['height']);
            if ($outPath == null) {
                $outPath = $this->image['path'];
            }
            switch (strtolower($this->image['ext']))
            { 
            	case 'jpg':
                   imagejpeg($thumb, $outPath .DS. $this->image['filename'], $this->jpeg_compress);
            	   break;
            	case 'png':
                   imagepng($thumb, $outPath .DS. $this->image['filename'], floor($this->jpeg_compress / 10));
            	   break;
            	case 'gif':
                   imagegif($thumb, $outPath .DS. $this->image['filename']);
            	   break;
            }
            $this->image['width'] = $new_width;
            $this->image['height'] = $new_height;
            $this->image['filesize'] = filesize($this->image['path'] .DS. $this->image['filename']);
        } else {
            if (($this->image['path'] != $outPath) and ($outPath != null)) {
                copy($this->image['path'] .DS. $this->image['filename'], $outPath .DS. $this->image['filename']);
            }    
        }
        $size = getimagesize($this->image['path'] .DS. $this->image['filename']);
        $this->image['width']  = $size[0];
        $this->image['height'] = $size[1];
    }

    function cropImage($cropdata, $size) {

        $size = explode('x', $size);
        $thumb = imagecreatetruecolor($size[0], $size[1]);
        imagealphablending( $thumb, false );
        imagesavealpha( $thumb, true );
        switch (strtolower($this->image['ext']))
        { 
        	case 'jpg':
               $source = imagecreatefromjpeg( $this->image['path'] .DS. $this->image['filename'] );
        	   break;
        	case 'gif':
               $source = imagecreatefromgif( $this->image['path'] .DS. $this->image['filename'] );
               imagealphablending( $source, true );
               imagesavealpha( $source, true );
        	   break;
        	case 'png':
               $source = imagecreatefrompng( $this->image['path'] .DS. $this->image['filename'] );
               imagealphablending( $source, false );
               imagesavealpha( $source, true );
        	   break;
        }

        imagecopyresampled($thumb, $source, 0, 0, $cropdata['left'], $cropdata['top'], $size[0], $size[1], $cropdata['width'], $cropdata['height']);

        switch (strtolower($this->image['ext']))
        { 
        	case 'jpg':
               imagejpeg($thumb, $this->image['path'] .DS. $this->image['filename'], $this->jpeg_compress);
        	   break;
        	case 'png':
               imagepng($thumb, $this->image['path'] .DS. $this->image['filename'], floor($this->jpeg_compress / 10));
        	   break;
        	case 'gif':
               imagegif($thumb, $this->image['path'] .DS. $this->image['filename']);
        	   break;
        }
        chmod($this->image['path'] .DS. $this->image['filename'], 0664); 

        $size = getimagesize($this->image['path'] .DS. $this->image['filename']);
        $this->image['width']  = $size[0];
        $this->image['height'] = $size[1];
        
    }

    function createThumb($size = '120x120', $dir = '_thumbs')
    {
        if (!$this->_gd) {
            $this->image['thumb'] = '0';
            return false;    
        }
        
        $size = $this->getOrientation($this->image['width'].'x'.$this->image['height'], $size);
        
        $size = explode('x', $size);

        $percent['w'] = $this->image['width'] / $size[0]; 
        $percent['h'] = $this->image['height'] / $size[1];
        
        if ($percent['h'] > $percent['w']) {
            $width  = $this->image['width'];
            $height = floor( $size[1] * $percent['w'] ); 
            $left = 0;
            switch ($this->_align['vertical']) { 
            	case 'top':
                   $top = 0;
            	   break;

            	case 'bottom':
                    $top = floor($this->image['height'] - $height);
            	    break;
    
            	default :
                    $top = floor(($this->image['height'] - $height) / 2);
            }
        } else {
            $height  = $this->image['height'];
            $width = floor( $size[0] * $percent['h'] );
            
            switch ($this->_align['horizontal']) { 
            	case 'left':
                    $left = 0;
     	            break;
            
            	case 'right':
                    $left = floor($this->image['width'] - $width);
            	break;
            
            	default :
                    $left = floor(($this->image['width'] - $width) / 2);
                    
            }            
            $top = 0;
        } 
    
        $thumb = imagecreatetruecolor($size[0], $size[1]);
        imagealphablending( $thumb, false );
        imagesavealpha( $thumb, true );
        switch (strtolower($this->image['ext']))
        { 
        	case 'jpg':
               $source = imagecreatefromjpeg( $this->image['path'] .DS. $this->image['filename'] );
        	   break;
        	case 'gif':
               $source = imagecreatefromgif( $this->image['path'] .DS. $this->image['filename'] );
               imagealphablending( $source, true );
               imagesavealpha( $source, true );
        	   break;
        	case 'png':
               $source = imagecreatefrompng( $this->image['path'] .DS. $this->image['filename'] );
               imagealphablending( $source, false );
               imagesavealpha( $source, true );
        	   break;
        }
        // imagecopyresized($thumb, $source, 0, 0, $left, $top, $size[0], $size[1], $width, $height);
        imagecopyresampled($thumb, $source, 0, 0, $left, $top, $size[0], $size[1], $width, $height);
        $this->setThumbsPath($dir);

        switch (strtolower($this->image['ext']))
        { 
        	case 'jpg':
               imagejpeg($thumb, $this->image['path'] .DS. $dir .DS. $this->image['filename'], $this->jpeg_compress);
        	   break;
        	case 'png':
               imagepng($thumb, $this->image['path'] .DS. $dir .DS. $this->image['filename'], floor($this->jpeg_compress / 10));
        	   break;
        	case 'gif':
               imagegif($thumb, $this->image['path'] .DS. $dir .DS. $this->image['filename']);
        	   break;
        }
        chmod($this->image['path'] .DS. $dir .DS. $this->image['filename'], 0664); 
        $this->image['thumb'] = '1';
        return $this->image['thumb'];
    }

    function checkThumb($dir)
    {
        if (is_file($this->image['path'] .DS. $dir .DS. $this->image['filename'])) {
            $this->image['thumb'] = '1';
        } else {
            $this->image['thumb'] = '0';
        }    
        return $this->image['thumb'];
    }

    public function getImageData()
    {
        return $this->image;
    }

    public function makeThumb($createDir = true)
    {
        if ($createDir) {
            // Ha kell létrehozzuk a bélyegképek könyvtárát
            // $this->setThumbsPath('_thumbs');
        }
    }
    
    public function setThumbDir($dir)
    {
        $this->image['thumbDir'] = $dir;
    }

    private function setThumbsPath($path = '')
    {
        if ($path != '') {
            $this->image['thumbDir'] = $path;
        }
        $path = $this->image['path'] .DS. $this->image['thumbDir'];
        $path = str_replace($_SERVER['DOCUMENT_ROOT'], '', $path);
        $path = explode(DS, $path);
        $tmp = $_SERVER['DOCUMENT_ROOT'];

        for ($i = 0; $i <= count($path)-1; $i++) {
            $tmp .= DS . $path[$i];
            if ( !file_exists($tmp) ) {
                mkdir($tmp);
                chmod($tmp, 0775); 
                $content = '<html><body bgcolor="#FFFFFF"></body></html>';
                $fp = fopen($tmp .DS. 'index.html','wb');
                fwrite($fp,$content);
                fclose($fp);
                chmod($tmp .DS. 'index.html', 0664); 
            }
        }
    }

    public function setOutPath($path)
    {
        if ( substr($path, -1) == '/' ) {$path = substr($path, 0, -1); }
        $this->output['file'] = $path .DS. $this->output['name'] . '.' . $this->output['ext'];
        $this->output['path'] = $path;   
    }
        
    public function getImage()
    {
        return $this->output['file'];
    }
    
    public function getSize()
    {
        $size = new stdClass();
        $size->width  = $this->image['width'];
        $size->height = $this->image['height'];
        return $size;
    }
    
    public function setOutput($file)
    {
        $this->outName = $file;
    }
    
    function createWatermark($image)
    {
        if (!$this->_gd) {
            return false;    
        }
        
        if (!file_exists($this->watermark['image'])) { return false; }
        
        if ($image != '') {
            $this->watermark['image'] = $image;    
            $size = getimagesize($this->watermark['image']);
            $this->watermark['width'] = $size[0];
            $this->watermark['height'] = $size[1];

            $this->watermark['dest_width'] = $this->watermark['width'];
            $this->watermark['dest_height'] = $this->watermark['height'];

            $ratioX = $this->watermark['width'] / $this->image['width'];
            $ratioY = $this->watermark['height'] / $this->image['height'];

            if (($ratioX > 0.5) or ($ratioY > 0.5)) {
                $ratioX = ($this->image['width'] / 2) / $this->watermark['width'];
                $ratioY = ($this->image['height'] / 2) / $this->watermark['height'];
                if ($ratioX < $ratioY) {
                    $ratio = $ratioX;
                } else {
                    $ratio = $ratioY;
                }
                $this->watermark['dest_width'] = $this->watermark['width'] * $ratio;
                $this->watermark['dest_height'] = $this->watermark['height'] * $ratio;
            }
        }
        if (($this->watermark['dest_width'] < 80) and ($this->watermark['dest_height'] < 80)) {
            if (($this->watermark['width'] > 80) and ($this->watermark['height'] > 80)) {
                return false;
            }
        }

        switch (strtolower($this->image['ext']))
        { 
            case 'jpg':
                $out = @imagecreatefromjpeg($this->image['path'] .DS. $this->image['filename']);
                break;
            
            case 'png':
                $out = @imagecreatefrompng($this->image['path'] .DS. $this->image['filename']);
                imagealphablending( $out, true );
                imagesavealpha( $out, true );
                break;
            
            case 'gif':
                $out = @imagecreatefromgif($this->image['path'] .DS. $this->image['filename']);
                imagealphablending( $out, true );
                imagesavealpha( $out, true );
                break;
        }
        $wm = @imagecreatefrompng($this->watermark['image']);
        
        // Pozícionálás beállítása
        switch ($this->watermark['walign']) { 
        	case 'left':
                $w = 10;
            	break;
        	case 'center':
                $w = ($this->image['width'] - $this->watermark['dest_width']) / 2; 
            	break;
        	case 'right':
                $w = $this->image['width'] - ($this->watermark['dest_width'] + 10); 
            	break;
        }
        switch ($this->watermark['halign']){ 
        	case 'top':
                $h = 10;
            	break;
        	case 'center':
                $h = ($this->image['height'] - $this->watermark['dest_height']) / 2; 
            	break;
        	case 'bottom':
                $h = $this->image['height'] - ($this->watermark['dest_height'] + 10); 
            	break;
        }
        imagecopyresampled($out, $wm, $w, $h, 0, 0, $this->watermark['dest_width'], $this->watermark['dest_height'], $this->watermark['width'], $this->watermark['height']);

        // A kimenet szükség esetén felülírható
        // echo $this->outName;
        switch (strtolower($this->image['ext']))
        { 
            case 'jpg':
                imagejpeg($out, $this->image['path'] .DS. $this->image['filename'], $this->jpeg_compress);
                break;
            
            case 'png':
                imagepng($out, $this->image['path'] .DS. $this->image['filename'], floor($this->jpeg_compress / 10));
                break;
            
            case 'gif':
                imagegif($out, $this->image['path'] .DS. $this->image['filename']);
                break;
        }
        imagedestroy($out);
        imagedestroy($wm);
        return true;
    }
}
?>